//+-----------------------------------------------------------------------------------------------------------------------------+
//| EXPERT:       ArunFX.mq5
//| APPLICATION:  EXPERT ADVISOR (MULTI SYMBOL) FOR ALGORITHEMIC TRADING
//+-----------------------------------------------------------------------------------------------------------------------------+
//+-----------------------------------------------------------------------------------------------------------------------------+
//| INCLUDE FILES:
//+-----------------------------------------------------------------------------------------------------------------------------+
	#include "../FrameWork.mqh"
//+-----------------------------------------------------------------------------------------------------------------------------+
//| CLASS(s) INSTANTIATION:
//+-----------------------------------------------------------------------------------------------------------------------------+
	CExpertBase 					*Expert_US30;
//+-----------------------------------------------------------------------------------------------------------------------------+
//| INPUT:  Input for variable used in the expert advisor
//| NOTE:   Indicator parameters defaluted in CBase class (PRICE_CLOSE,VOLUME_TICK)
//+-----------------------------------------------------------------------------------------------------------------------------+
	input	group "TRADED SYMBOL(s)"
				string								InpSymbol1 					= "US30";						// EXPERT's SYMBOL1
				string								InpSymbol2 					= "GER40";					// EXPERT's SYMBOL2
	input	group "MONEY MANAGEMENT"
				double								RiskPerSymbol 	 		= 2.00;							// TOTAL RISK PERCENT PER SYMBOL, ENTER AS 1.25 FOR 1.25%
				double								RiskPerTrade 	 			= 1.00;							// RISK PERCENT PER TRADE, ENTER AS 0.50 FOR 0.05%
				double  							InitialDeposit			= 5000;							// INITIAL DEPOSIT IN TRADE ACCOUNT
				double  							MinEquityBalance	 	= 4000;							// MINIMUM EQUITY BALANCE TO BE MAINTAINED
	input	group "INDICATOR PARAMETERS: iAroon"
				int										Aroon_Period 	 			= 13;								// NO OF BARS TO LOOK BACK FOR HIGHEST / LOWEST PRICE
				int										Aroon_ThldUP 	 			= 70;								// UPPER THRESHOLD LIMIT TO IDENTIFY STRONG TREND

//+-----------------------------------------------------------------------------------------------------------------------------+
//| EXPERT:       OnInit() Initialization Function
//+-----------------------------------------------------------------------------------------------------------------------------+
int OnInit() {

		// VALIDATE RISK PERCENT INPUTS
		if(RiskPerTrade <= 0.00) {
			PrintFormat("RISK PER TRADE [%.2f] SHOULD BE GREATER THEN ZERO. EA WILL NOT RUN",RiskPerTrade);
			return(INIT_PARAMETERS_INCORRECT);
		}

	//+-----------------------------------------------------------------------------------------------------------------------------+
	//| SET UP EXPERT FOR US30 (US Wall Street 30 Index)
	//+-----------------------------------------------------------------------------------------------------------------------------+
		Expert_US30 = new CExpertBase(InpSymbol1,RiskPerSymbol,RiskPerTrade);
		Expert_US30.SetParamAroon(Aroon_Period,Aroon_ThldUP);
	//+-----------------------------------------------------------------------------------------------------------------------------+
		return(INIT_SUCCEEDED);

} // END of function OnInit()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| EXPERT:       OnDeinit() DeInitialization Function
//+-----------------------------------------------------------------------------------------------------------------------------+
void OnDeinit(const int reason) {

		delete Expert_US30;
   
} // END of function OnDeinit()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| EXPERT:       OnTick() Tick Function
//+-----------------------------------------------------------------------------------------------------------------------------+
void OnTick() {

	// CHECK & EXIT, IF TRADING IS NOT ALLOWED
  if(!TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))    												return;
  if(!MQLInfoInteger(MQL_TRADE_ALLOWED))              												return;
  if(!AccountInfoInteger(ACCOUNT_TRADE_EXPERT))       												return;
  if(!AccountInfoInteger(ACCOUNT_TRADE_ALLOWED))      												return;

	Expert_US30.OnTick();

} // END of function OnTick()
//+-----------------------------------------------------------------------------------------------------------------------------+

