#define  APP_COPYRIGHT	"Copyright 2023 Anil Varma"
#define  APP_LINK				"AnilHVarma@gmail.com"
#define  APP_VERSION		"1.00"

#include <Object.mqh>
#include "../../FrameWork.mqh"

#define _INIT_CHECK_FAIL 													\
   if(mInitResult != INIT_SUCCEEDED ) 						return (mInitResult);
#define _INIT_ERROR(msg)													return ( InitError(msg,INIT_PARAMETERS_INCORRECT));
#define _INIT_ASSERT(condition,msg) 							\
   if(!condition) 																return ( InitError(msg,INIT_FAILED) );
//+-----------------------------------------------------------------------------------------------------------------------------+
//| CLASS:        CCommonBase.mqh
//| APPLICATION:  COMMON BASE CLASS FOR SETTING UP TRADE AND INDICATOR PARAMETERS
//+-----------------------------------------------------------------------------------------------------------------------------+
class CCommonBase : public CObject {

	//+---------------------------------------------------------------------------------------------------------------------------+
	//| PARAMETRIC CONSTRUCTOR METHOD TO BE CALLED FROM CExpertBase CLASS TO SET ALL TRADE AND INDICATOR PARAMETERS AS GLOBAL
	//+---------------------------------------------------------------------------------------------------------------------------+
	public:									// TO BE CALLED FROM CStrategyBase CLASS AND SET PARAMETERS
		CCommonBase(string pSymbol,double pRiskPerSymbol,double pRiskPerTrade,
								int pAroon_Period,int pAroon_ThldUP);
	 ~CCommonBase();
		CCommonBase(string pSymbol) { mSymbol = pSymbol; }
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| ERROR HANDLING
	//+---------------------------------------------------------------------------------------------------------------------------+
	protected:	// METHODS
		string          			mInitMessage;
		int             			mInitResult;
		int 									InitError(string initMessage,int initResult);

	public:			// PROPERTIES
		int    								InitResult() 	{ return(mInitResult); }
		string 								InitMessage() { return(mInitMessage); }
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| MEMBER VARIABLES: SYMBOL AND RELATED INFORMATION
	//+---------------------------------------------------------------------------------------------------------------------------+
	protected:
	  string				      	mSymbol;
	  int					      		mDigits;
		ENUM_APPLIED_VOLUME		mAppliedVolume;
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| MEMBER VARIABLES: RISK MANAGEMENT
	//+---------------------------------------------------------------------------------------------------------------------------+
		double								mRiskPerSymbol;
		double								mRiskPerTrade;
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| MEMBER VARIABLES: CHART IDs FOR CREATING OBJECTS
	//+---------------------------------------------------------------------------------------------------------------------------+
		int 									mChartID;
		int										mSubWindowID;
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| FILTER TREND: LOAD INDICATOR MEMBER VARIABLES
	//+---------------------------------------------------------------------------------------------------------------------------+
	protected:
		// INDICATOR: iAroon
		int										mAroon_Period;										// NO OF BARS TO LOOK BACK FOR HIGHEST/LOWEST PRICES
		int										mAroon_ThldUP;										// UPPER THRESHOLD TO IDENTIFY STRONG TREND
		int										mAroon_ThldMP;										// MID POINT OF RANGE BOUND INDICATOR MP = (MAX 100 + MIN 0) / 2
		int										mAroon_ThldDN;										// LOWER THRESHOLD TO IDENTIFY RANGING MARKET

}; // END of Class declaration
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       CCommonBase()
//| APPLICATION:  CLASS INITIALIAZATION METHOD TO TAKE ALL PARAMETERS INCLUDING THE ONE FOR INDICATOR(s)
//| NOTE:					TO BE CALLED ONCE FROM METHOD CStrategyBase.Init() IN CStrategyBase CLASS
//+-----------------------------------------------------------------------------------------------------------------------------+
CCommonBase::CCommonBase(string pSymbol,double pRiskPerSymbol,double pRiskPerTrade,
												 int pAroon_Period,int pAroon_ThldUP) {

  	// ASSIGN / DEFAULT MEMBER VARIABLE VALUES
	  mSymbol = pSymbol;
	  mDigits = (int)SymbolInfoInteger(mSymbol,SYMBOL_DIGITS);
		mAppliedVolume = VOLUME_TICK; // SET DEFAULT APPLIED VOLUME TYPE (GetVolume() USES iVolumes() WHICH RETURNS ONLY TICK VOLUME)

		// RISK MANAGEMENT VARIABLES
		mRiskPerSymbol	= NormalizeDouble(pRiskPerSymbol/100,3);				// RISK @1.50% SAVED AS 0.015
		mRiskPerTrade		= NormalizeDouble(pRiskPerTrade/100,3);					// RISK @0.50% SAVED AS 0.005
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| NOTE:  Parameter values defined in PARAMETRIC CONSTRUCTOR are available globally to the derived class(s). Thus avoid need
	//|				 to redefine them again on each derived class. As agaist using SetParam() which is not part of constructor.
	//+---------------------------------------------------------------------------------------------------------------------------+
		//+---------------------------------------------------------------------------------------------------------------------------+
		//| FILTER TREND: iAroon INDICATOR
		//+---------------------------------------------------------------------------------------------------------------------------+
			mAroon_Period = pAroon_Period;
			mAroon_ThldUP = pAroon_ThldUP;
			mAroon_ThldMP = (100 + 0) / 2;
			mAroon_ThldDN = 100 - mAroon_ThldUP;
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| HARDLY ANYTHING GO WRONG HERE, WE RETURN
	//+---------------------------------------------------------------------------------------------------------------------------+
		InitError("",INIT_SUCCEEDED);
		PrintFormat("[%s] %s mInitResult[%i] mInitMessage[%s]",(string)mSymbol,(string)__FUNCTION__,mInitResult,mInitMessage);
		//return(INIT_SUCCEEDED);

} // END of method CCommonBase()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       ~CCommonBase()
//| APPLICATION:  
//+-----------------------------------------------------------------------------------------------------------------------------+
CCommonBase::~CCommonBase(void) {


} // END of method ~CCommonBase()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       InitError()
//| APPLICATION:  
//+-----------------------------------------------------------------------------------------------------------------------------+
int CCommonBase::InitError(string initMessage,int initResult) {

		mInitMessage = initMessage;
		mInitResult  = initResult;

		if(initMessage != "") {
			PrintFormat("[%s] %s  initResult[%i]  initMessage[%s]",(string)mSymbol,(string)__FUNCTION__,initResult,initMessage);
		}

		return(initResult);

} // END of method InitError()
//+-----------------------------------------------------------------------------------------------------------------------------+
