#include "StrategyBase.mqh"
//+-----------------------------------------------------------------------------------------------------------------------------+
//| CLASS:        CExpertBase.mqh
//| APPLICATION:  EXPERT BASE CLASS FOR TRADING MULTIPLE SYMBOLS BY CREATING INSTANCE FOR EACH SYMBOL
//+-----------------------------------------------------------------------------------------------------------------------------+
class CExpertBase {

	//+---------------------------------------------------------------------------------------------------------------------------+
	//| CONSTRUCTOR AND DECONSTRUCTOR IN PROTECTED SECTION, ALLOWING ONLY DERIVED CLASSES TO ACCESS THEM
	//+---------------------------------------------------------------------------------------------------------------------------+
	public:
		// PARAMETRIC CONSTRUCTOR TO SET CORRECT PARAMETERS WITH new CLASS METHOD
		CExpertBase(string pSymbol,double pRiskPerSymbol,double pRiskPerTrade);
	 ~CExpertBase();
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| REPLICA OF EXPERT ADVISOR METHOD(s) CALLED FROM THE EXPERT ADVISOR
	//+---------------------------------------------------------------------------------------------------------------------------+
	public:
		int 									OnInit();
		void									OnDeinit(const int reason) 		{ return; }
		void 									OnTick();
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| CExpertBase CLASS WILL HAVE ITS OWN IsNewBar METHODS, AS IT IS NOT DERIVED CLASS OF CStrategyBase
	//| WE KEEP THEM AS PRIVATE AS MULTIPLE INSTANCE OF THIS CLASS WILL BE CREATED FOR DIFFERENT SYMBOL(s)
	//+---------------------------------------------------------------------------------------------------------------------------+
	private:
		string								vSymbol;
		double								vRiskPerSymbol;
		double								vRiskPerTrade;
	  ENUM_TIMEFRAMES	    	vTFD01;				bool	IsNewBarD01();				datetime	vPrevTimeD01;
	  ENUM_TIMEFRAMES	    	vTFH01;				bool	IsNewBarH01();				datetime	vPrevTimeH01;				void RefreshBuffersH01(void);
	  ENUM_TIMEFRAMES	    	vTFM15;				bool	IsNewBarM15();				datetime	vPrevTimeM15;				void RefreshBuffersM15(void);
	  ENUM_TIMEFRAMES	    	vTFM05;				bool	IsNewBarM05();				datetime	vPrevTimeM05;				void RefreshBuffersM05(void);
	  ENUM_TIMEFRAMES	    	vTFM01;				bool	IsNewBarM01();				datetime	vPrevTimeM01;				void RefreshBuffersM01(void);
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| LOAD / INSTANTIATE CLASS(s)
	//+---------------------------------------------------------------------------------------------------------------------------+
		CStrategyBase					*cStrategyBase;
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| FILTER TREND: LOAD INDICATOR MEMBER VARIABLES
	//+---------------------------------------------------------------------------------------------------------------------------+
	private:
		// INDICATOR: iAroon
		int										vAroon_Period;										// NO OF BARS TO LOOK BACK FOR HIGHEST/LOWEST PRICES
		int										vAroon_ThldUP;										// UPPER THRESHOLD TO IDENTIFY STRONG TREND
	public:
		void SetParamAroon(int pAroon_Period,int pAroon_ThldUP);

}; // END of Class declaration
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       CExpertBase()
//| APPLICATION:  CONSTRUCTOR METHOD WILL CALL CCommonBase CLASS's CONSTRUCTOR METHOD AND PASS IN ALL PARAMETERS
//+-----------------------------------------------------------------------------------------------------------------------------+
CExpertBase::CExpertBase(string pSymbol,double pRiskPerSymbol,double pRiskPerTrade) {

		// CLASS NEEDS ITS OWN VARIABLES ASSIGNED WITH VALUES TO OPERATE IsNewBar()s
		vSymbol 			 = pSymbol;
		vRiskPerSymbol = pRiskPerSymbol;
		vRiskPerTrade	 = pRiskPerTrade;
		vPrevTimeD01	= WRONG_VALUE;
		vPrevTimeH01	= WRONG_VALUE;
		vPrevTimeM15	= WRONG_VALUE;
		vPrevTimeM05	= WRONG_VALUE;
		vPrevTimeM01	= WRONG_VALUE;
		// SET TIME FRAME VALUES
	  vTFD01	= PERIOD_D1;
	  vTFH01	= PERIOD_H1;
	  vTFM15  = PERIOD_M15;
	  vTFM05	= PERIOD_M5;
	  vTFM01	= PERIOD_M1;

} // END of method CExpertBase()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       ~CExpertBase()
//| APPLICATION:  
//+-----------------------------------------------------------------------------------------------------------------------------+
CExpertBase::~CExpertBase(void) {

		delete	cStrategyBase;

} // END of method ~CExpertBase()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       OnInit()
//| APPLICATION:  
//+-----------------------------------------------------------------------------------------------------------------------------+
int CExpertBase::OnInit(void) {

		cStrategyBase = new CStrategyBase(vSymbol,vRiskPerSymbol,vRiskPerTrade,
																			vAroon_Period,vAroon_ThldUP);	

		return(INIT_SUCCEEDED);

} // END of method OnInit()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       OnTick()
//| APPLICATION:  
//+-----------------------------------------------------------------------------------------------------------------------------+
void CExpertBase::OnTick(void) {

		cStrategyBase.OnTick();

} // END of method OnTick()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:     IsNewBarD01()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CExpertBase::IsNewBarD01() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(vSymbol,vTFD01,SERIES_LASTBAR_DATE);

  if(vPrevTimeD01 != vCurrTime)			{ vPrevTimeD01 = vCurrTime;		return(true); }
  return(false);

} // END of method IsNewBarD01()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:      IsNewBarH01()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CExpertBase::IsNewBarH01() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(vSymbol,vTFH01,SERIES_LASTBAR_DATE);

  if(vPrevTimeH01 != vCurrTime) 		{ vPrevTimeH01 = vCurrTime;		return(true); } 
  return(false);

} // End of method IsNewBarH01()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:      IsNewBarM15()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CExpertBase::IsNewBarM15() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(vSymbol,vTFM15,SERIES_LASTBAR_DATE);

  if(vPrevTimeM15 != vCurrTime) 		{ vPrevTimeM15 = vCurrTime;		return(true); }  
  return(false);

} // End of method IsNewBarM15()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:      IsNewBarM05()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CExpertBase::IsNewBarM05() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(vSymbol,vTFM05,SERIES_LASTBAR_DATE);

  if(vPrevTimeM05 != vCurrTime) 		{ vPrevTimeM05 = vCurrTime;		return(true); }  
  return(false);

} // End of method IsNewBarM05()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:      IsNewBarM01()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CExpertBase::IsNewBarM01() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(vSymbol,vTFM01,SERIES_LASTBAR_DATE);

  if(vPrevTimeM01 != vCurrTime) 		{ vPrevTimeM01 = vCurrTime;		return(true); }
  return(false);

} // End of method IsNewBarM01()
//+-----------------------------------------------------------------------------------------------------------------------------+

/***** METHOD TO SET PARAMETER VALUES *****/
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:      IsNewBarM01()
//+-----------------------------------------------------------------------------------------------------------------------------+
void CExpertBase::SetParamAroon(int pAroon_Period,int pAroon_ThldUP) {

		vAroon_Period = pAroon_Period;
		vAroon_ThldUP = pAroon_ThldUP;

} // End of method IsNewBarM01()
//+-----------------------------------------------------------------------------------------------------------------------------+
