#include "CommonBase.mqh"
//+-----------------------------------------------------------------------------------------------------------------------------+
//| CLASS:        CStrategyBase.mqh
//| APPLICATION:  STRATEGY BASE CLASS WITH COMMON TRADE OPERATION METHODS
//+-----------------------------------------------------------------------------------------------------------------------------+
class CStrategyBase : protected CCommonBase {

	//+---------------------------------------------------------------------------------------------------------------------------+
  //| CONSTRUCTOR AND DECONSTRUCTOR
	//+---------------------------------------------------------------------------------------------------------------------------+
	public:
		// PARAMETRIC CONSTRUCTOR TO BE CALLED FROM CExpertBase CLASS AND TO SET PARAMETERS IN CCommonBase CLASS
		CStrategyBase(string pSymbol,double pRiskPerSymbol,double pRiskPerTrade,
									int pAroon_Period,int pAroon_ThldUP);
		// DEFAULT DECONSTRUCTOR METHOD
	 ~CStrategyBase();
		// DEFAULT CONSTRUCTOR TO BE CALLED FROM INDIVIDUAL STRATEGY CLASS(s) TO ACCESS GLOBAL PARAMETER VALUES FROM CCommonBase
		CStrategyBase(string pSymbol) : CCommonBase(pSymbol) { }

	public:
		virtual int 					OnInit();
		virtual void					OnDeinit(const int reason) 		{ return; }
		virtual void 					OnTick();
	//+---------------------------------------------------------------------------------------------------------------------------+
  //| NEW CONNECTION TO BROKER FOR FIRST TIME
	//+---------------------------------------------------------------------------------------------------------------------------+
	protected:
		bool                	mFirstTime;												// REPLACE WITH mConnected ... RECEIVED FIRST TICK AFTER CONNECTION
		MqlTick             	mLastTick;
		virtual void        	RunSetup() { return; }
	//+---------------------------------------------------------------------------------------------------------------------------+
  //| POSITIONS COUNT
	//+---------------------------------------------------------------------------------------------------------------------------+
	protected:
		int										mMAXPosSymbol;										// MAXIMUM POSITIONS OPEN AT A TIME FOR THE 'SYMBOL''
		int										mMAXPosMagic;											// MAXIMUM POSITIONS OPEN AT A TIME FOR THE 'SYMBOL AND MAGIC'
	//+---------------------------------------------------------------------------------------------------------------------------+
  //| LOT SIZE
	//+---------------------------------------------------------------------------------------------------------------------------+
	private:
	  double              	mLotSize;             						// TRADING VOLUME OF POSITION
	  int										mLotDigits;           						// NO OF DIGITS IN THE LOT SIZE OF SYMBOL
	  double  							mMinLotSize;  										// MINIMUM VOLUME (LOT SIZE) OF THE SYMBOL
	//+---------------------------------------------------------------------------------------------------------------------------+
  //| OPEN POSITIONS
	//+---------------------------------------------------------------------------------------------------------------------------+
	protected:
	  double    						Ask()    			{ return(NormalizeDouble(SymbolInfoDouble(mSymbol,SYMBOL_ASK),mDigits)); }
	  double    						Bid()					{ return(NormalizeDouble(SymbolInfoDouble(mSymbol,SYMBOL_BID),mDigits)); }

		// Highest AND Lowest INDEX AND PRICE FOR PREVIOUS pCount BARS from pIndex
		int										mIdxHighestH01,			mIdxHighestM15,		mIdxHighestM05,		mIdxHighestM01;
		int										mIdxLowestH01,			mIdxLowestM15,		mIdxLowestM05,		mIdxLowestM01;

		int										HighestIdx(ENUM_TIMEFRAMES pTimeFrame,int pCount,int pIndex) 
																		{ return(iHighest(mSymbol,pTimeFrame,MODE_HIGH,pCount,pIndex)); }
		double								Highest(ENUM_TIMEFRAMES pTimeFrame,int pCount,int pIndex)
						 												{ return(iHigh(mSymbol,pTimeFrame,iHighest(mSymbol,pTimeFrame,MODE_HIGH,pCount,pIndex))); }

		int										LowestIdx(ENUM_TIMEFRAMES pTimeFrame,int pCount,int pIndex)
																		{ return(iLowest(mSymbol,pTimeFrame,MODE_LOW,pCount,pIndex)); }
		double								Lowest(ENUM_TIMEFRAMES pTimeFrame,int pCount,int pIndex)
						 												{ return(iLow(mSymbol,pTimeFrame,iLowest(mSymbol,pTimeFrame,MODE_LOW,pCount,pIndex))); }

	  int       						ConvertToPoints(double pPips) 		{ return(int)(pPips / SymbolInfoDouble(mSymbol,SYMBOL_POINT));   }
	  double    						ConvertToPips(int pPoints)    		{ return     (pPoints * SymbolInfoDouble(mSymbol,SYMBOL_POINT)); }

	  double              	mPriceLimit;
	  double              	mPriceEntry;
	  double              	mPriceSL;
	  double              	mPriceTP;
	  int                 	mSetOffPoints;        						// BUFFER POINTS ADDED / SUBTRACTED FROM STOP LOSS PRICE
	  double              	mSetOff;              						// BUFFER POINTS CONVERTED TO PRICE VALUE

		int										mStopLevel;												// Minimal indention in points from the current close price to place Stop orders
		int										mMinSLPoints;											// MINIMUM STOP LOSS IN POINTS TO BE MAINTAINED ON PLACING ORDER
		double								mMinSLPips;												// MINIMUM STOP LOSS IN POINTS CONVERTED TO SYMBOL PIPS
	  int                 	mCount;               						// STANDARD NUMBER OF BAR's FETCHED IN GetArray(mCount) CALL

	protected:
		// MEMBER VARIABLES FOR MAGIC NO OF DIFFERENT STRATEGY
	  int                 	mMagic;
	  // MEMBER VARIABLES FOR COMMENTS
	  string              	mComment;
	  //string              	mCommentEntry;
	  //string              	mCommentOpen;
	  string              	mCommentClose;
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| BASIC TIME FRAME USED, ARE DEFAULTED WITHOUT PASSING IN AS PARAMETERS
	//+---------------------------------------------------------------------------------------------------------------------------+
	protected:
	  ENUM_TIMEFRAMES	    	mTFD01;
	  ENUM_TIMEFRAMES	    	mTFH01;
	  ENUM_TIMEFRAMES	    	mTFM15;
	  ENUM_TIMEFRAMES	    	mTFM05;
	  ENUM_TIMEFRAMES	    	mTFM01;
	//+---------------------------------------------------------------------------------------------------------------------------+
  //| NEW BAR FORMATION: PRIVATE TO CStrategyBase INSTANCE FOR MULTI SYMBOL OR TIME FRAMES USED
	//+---------------------------------------------------------------------------------------------------------------------------+
	protected:
		virtual bool					IsNewBarD01();						datetime	mPrevTimeD01;
		virtual bool					IsNewBarH01();						datetime	mPrevTimeH01;
		virtual bool					IsNewBarM15();						datetime	mPrevTimeM15;
		virtual bool					IsNewBarM05();						datetime	mPrevTimeM05;
		virtual bool					IsNewBarM01();						datetime	mPrevTimeM01;

}; // END Of Class DEFINITION
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:  		CStrategyBase()
//+-----------------------------------------------------------------------------------------------------------------------------+
CStrategyBase::CStrategyBase(string pSymbol,double pRiskPerSymbol,double pRiskPerTrade,
														 int pAroon_Period,int pAroon_ThldUP)
		: CCommonBase(pSymbol,pRiskPerSymbol,pRiskPerTrade,
									pAroon_Period,pAroon_ThldUP) {

		string vFunction = "[" + (string)mSymbol + "] " + (string)__FUNCTION__;

		// IF ERROR IN INITIALIZING CBaseCommon CLASS
		if(InitResult() != INIT_SUCCEEDED) {
			PrintFormat("%s InitResult[%i] InitMessage[%s]",vFunction,InitResult(),InitMessage());
			// return(InitResult()); ... Gives Error 'void' function returns a value
		}
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| SET TIME FRAME VALUES
	//+---------------------------------------------------------------------------------------------------------------------------+
	  mTFD01	= PERIOD_D1;					mPrevTimeD01 = WRONG_VALUE;
	  mTFH01	= PERIOD_H1;					mPrevTimeH01 = WRONG_VALUE;
	  mTFM15  = PERIOD_M15;					mPrevTimeM15 = WRONG_VALUE;
	  mTFM05	= PERIOD_M5;					mPrevTimeM05 = WRONG_VALUE;
	  mTFM01	= PERIOD_M1;					mPrevTimeM01 = WRONG_VALUE;

		mMAXPosSymbol = 1;
		mMAXPosMagic 	= 1;
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| SET NUMBER OF DIGITS FOR THE SYMBOL LOT (VOLUME)
	//+---------------------------------------------------------------------------------------------------------------------------+
	  mMinLotSize = SymbolInfoDouble(mSymbol,SYMBOL_VOLUME_MIN);  			// MINIMAL VOLUME FOR A POSITION
	  if(mMinLotSize 			== 0.001)  mLotDigits = 3;										// CALCULATE and ASSIGN LotSize DIGITS PARAMETERS
	  else if(mMinLotSize == 0.01)   mLotDigits = 2;
	  else if(mMinLotSize == 0.1)    mLotDigits = 1;
	  else Alert("%s: Failed calculate mLotDigits",vFunction);
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| SET MINIMUM STOP LOSS POINTS / PIPS FOR THE SYMBOL
	//+---------------------------------------------------------------------------------------------------------------------------+
		mStopLevel 	 = (int)SymbolInfoInteger(mSymbol,SYMBOL_TRADE_STOPS_LEVEL);
	  mMinSLPoints = 98;																								// MAX SPREAD FOR US30 IS 98 POINTS WHICH WE TAKE AS MIN SL
	  mMinSLPips 	 = ConvertToPips(mMinSLPoints);

		mSetOffPoints = 24;																								// MIN SPREAD
		mSetOff				= ConvertToPips(mSetOffPoints);

}; // END Of method CStrategyBase()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| EXPERT:       OnInit() Tick Function
//+-----------------------------------------------------------------------------------------------------------------------------+
CStrategyBase::~CStrategyBase(void) {


} // END of function OnInit()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| EXPERT:       OnTick() Tick Function
//+-----------------------------------------------------------------------------------------------------------------------------+
void CStrategyBase::OnTick() {

		// SET UP (LOADING DATA) IS DONE HERE ONLY WHEN IT IS FIRST TIME CONNECTION
		if(mFirstTime) {
			RunSetup();
		}

		// IF NOT CONNECTED TO BROKER SERVER, EXIT AND TRY AGAIN
		if(!SymbolInfoTick(mSymbol,mLastTick)) 																																							return;

		PrintFormat("Symbol[%s] AroonPeriod[%i] ThldUP[%i] ThldDN[%i]",mSymbol,mAroon_Period,mAroon_ThldUP,mAroon_ThldDN);


} // END of function OnTick()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:     IsNewBarD01()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CStrategyBase::IsNewBarD01() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(mSymbol,mTFD01,SERIES_LASTBAR_DATE);

  if(mPrevTimeD01 != vCurrTime)			{ mPrevTimeD01 = vCurrTime;		return(true); }
  return(false);

} // END of method IsNewBarD01()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:      IsNewBarH01()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CStrategyBase::IsNewBarH01() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(mSymbol,mTFH01,SERIES_LASTBAR_DATE);

  if(mPrevTimeH01 != vCurrTime) 		{ mPrevTimeH01 = vCurrTime;		return(true); } 
  return(false);

} // End of method IsNewBarH01()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:      IsNewBarM15()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CStrategyBase::IsNewBarM15() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(mSymbol,mTFM15,SERIES_LASTBAR_DATE);

  if(mPrevTimeM15 != vCurrTime) 		{ mPrevTimeM15 = vCurrTime;		return(true); }  
  return(false);

} // End of method IsNewBarM15()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:      IsNewBarM05()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CStrategyBase::IsNewBarM05() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(mSymbol,mTFM05,SERIES_LASTBAR_DATE);

  if(mPrevTimeM05 != vCurrTime) 		{ mPrevTimeM05 = vCurrTime;		return(true); }  
  return(false);

} // End of method IsNewBarM05()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:      IsNewBarM01()
//+-----------------------------------------------------------------------------------------------------------------------------+
bool CStrategyBase::IsNewBarM01() {

  datetime vCurrTime = (datetime)SeriesInfoInteger(mSymbol,mTFM01,SERIES_LASTBAR_DATE);

  if(mPrevTimeM01 != vCurrTime) 		{ mPrevTimeM01 = vCurrTime;		return(true); }
  return(false);

} // End of method IsNewBarM01()
//+-----------------------------------------------------------------------------------------------------------------------------+
